<?php
namespace codeneric\phmm {
  require_once ($GLOBALS["HACKLIB_ROOT"]);
  use \codeneric\phmm\base\includes\Error;
  use \codeneric\phmm\base\globals\Superglobals;
  class Utils {
    public static function admin_user_ids() {
      $wpdb = Superglobals::Globals("wpdb");
      \HH\invariant(
        $wpdb instanceof \wpdb,
        "%s",
        new Error("Can not get global wpdb object!")
      );
      $wp_user_search = $wpdb->get_results(
        "SELECT ID, display_name FROM ".$wpdb->users." ORDER BY ID"
      );
      $adminArray = array();
      foreach ($wp_user_search as $userid) {
        $curID = $userid->ID;
        $curuser = get_userdata($curID);
        $user_level = $curuser->user_level;
        if ($user_level >= 8) {
          $adminArray[] = $curID;
        }
      }
      return $adminArray;
    }
    public static function is_current_user_admin() {
      return current_user_can("administrator");
    }
    public static function string_contains($haystack, $needle) {
      $strpos = strpos($haystack, $needle);
      return $strpos !== false;
    }
    private static function generate_unique_id() {
      $uniqid = uniqid("", true);
      $uniqid = str_replace(".", "", $uniqid);
      return $uniqid;
    }
    static function get_plugin_id() {
      $id = get_option("cc_photo_manage_id", null);
      if (!\hacklib_cast_as_boolean(is_string($id))) {
        $id = self::generate_unique_id();
        update_option("cc_photo_manage_id", $id);
      }
      return $id;
    }
    static function apply_filter_or($filterHandle, $arg, $or) {
      if (\hacklib_cast_as_boolean(has_filter($filterHandle))) {
        return apply_filters($filterHandle, $arg);
      } else {
        return $or;
      }
    }
    static function wp_version_is_at_least($version) {
      $actual = get_bloginfo("version");
      return version_compare($actual, $version, ">=");
    }
    static function php_version_is_at_least($version) {
      $actual = phpversion();
      return version_compare($actual, $version, ">=");
    }
    static function wp_version_is_lower_than($version) {
      $actual = get_bloginfo("version");
      return version_compare($actual, $version, "<");
    }
    static function array_nested_key_exist($path, $array, $separator = ".") {
      if (!\hacklib_cast_as_boolean(is_array($array))) {
        return false;
      }
      $paths = explode($separator, $path);
      $p = array_shift($paths);
      if (\hacklib_cast_as_boolean(array_key_exists($p, $array))) {
        $arr = $array[$p];
        $path = implode($separator, $paths);
        if ($path === "") {
          return true;
        }
        return self::array_nested_key_exist($path, $arr, $separator);
      }
      return false;
    }
    static function get_temp_file($prefix) {
      $tmp_dir = sys_get_temp_dir();
      $temp_file = tempnam($tmp_dir, $prefix);
      \HH\invariant(
        is_string($temp_file),
        "%s",
        new Error("Unable to get a filename for a temporary file.")
      );
      $handle = fopen($temp_file, "w");
      \HH\invariant(
        is_resource($handle),
        "%s",
        new Error("Could not create temporary file.")
      );
      return array("resource" => $handle, "name" => $temp_file);
    }
    static function close_and_delete_file($handle, $name) {
      return
        \hacklib_cast_as_boolean(fclose($handle)) &&
        \hacklib_cast_as_boolean(unlink($name));
    }
    static function get_current_user_id() {
      return get_current_user_id();
    }
    static function get_post_meta_ONLY_USE_IN_HELPER_FUNCTIONS($postID, $key) {
      $meta = get_post_meta($postID, (string) $key, true);
      if ($meta === "") {
        return null;
      } else {
        return $meta;
      }
    }
    static function get_uncropped_image_sizes() {
      $_wp_additional_image_sizes = wp_get_additional_image_sizes();
      $sizes = array();
      foreach (get_intermediate_image_sizes() as $_size) {
        if (\hacklib_cast_as_boolean(
              in_array(
                $_size,
                array("thumbnail", "medium", "medium_large", "large")
              )
            )) {
          $sizes[$_size] = array("width" => 0, "height" => 0, "crop" => true);
          $sizes[$_size][\hacklib_id("width")] =
            (int) get_option($_size."_size_w");
          $sizes[$_size][\hacklib_id("height")] =
            (int) get_option($_size."_size_h");
          $sizes[$_size][\hacklib_id("crop")] =
            (bool) \hacklib_cast_as_boolean(get_option($_size."_crop"));
        } else {
          if (\hacklib_cast_as_boolean(
                is_array($_wp_additional_image_sizes)
              ) &&
              \hacklib_cast_as_boolean(
                array_key_exists($_size, $_wp_additional_image_sizes)
              )) {
            $sizes[$_size] = array(
              "width" =>
                (int) $_wp_additional_image_sizes[$_size][\hacklib_id(
                  "width"
                )],
              "height" =>
                (int) $_wp_additional_image_sizes[$_size][\hacklib_id(
                  "height"
                )],
              "crop" => (bool) \hacklib_cast_as_boolean(
                $_wp_additional_image_sizes[$_size][\hacklib_id("crop")]
              )
            );
          }
        }
      }
      $sizes = array_filter(
        $sizes,
        function($s) {
          return !\hacklib_cast_as_boolean($s[\hacklib_id("crop")]);
        }
      );
      return $sizes;
    }
  }
}
